"""``django-structlog`` is a structured logging integration for ``Django`` project using ``structlog``."""

name = "django_structlog"

VERSION = (10, 0, 0)

__version__ = ".".join(str(v) for v in VERSION)
