{% load static i18n %}<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="x-ua-compatible" content="ie=edge">
    <title>{% block title %}django_structlog_demo_project{% endblock title %}</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="">
    <meta name="author" content="">

    <link rel="icon" href="{% static 'images/favicons/favicon.ico' %}">

    {% block css %}

    <!-- Latest compiled and minified Bootstrap CSS -->
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-1BmE4kWBq78iYhFldvKuhfTAU6auU8tT94WrHftjDbrCEXSU1oBoqyl2QvZ6jIW3" crossorigin="anonymous">


    <!-- Your stuff: Third-party CSS libraries go here -->

    <!-- This file stores project-specific CSS -->

    <link href="{% static 'css/project.css' %}" rel="stylesheet">

    {% endblock %}

  </head>

  <body>


  <ul class="nav nav-tabs bg-light">
    <li class="nav-item">
      <a class="nav-link {% if "8000" in request.get_host|urlize %} active {% endif %}" href="http://localhost:8000">runserver_plus</a>
    </li>
    <li class="nav-item">
      <a class="nav-link {% if "8001" in request.get_host|urlize %} active {% endif %}" href="http://localhost:8001">wsgi</a>
    </li>
    <li class="nav-item">
      <a class="nav-link {% if "8002" in request.get_host|urlize %} active {% endif %}" href="http://localhost:8002">asgi</a>
    </li>
  </ul>
    <div class="mb-1">
      <nav class="navbar navbar-expand-md navbar-light">


        <div class="collapse navbar-collapse" id="navbarSupportedContent">

          <ul class="navbar-nav mr-auto">
            <li class="nav-item">
              <a class="nav-link navbar-brand" href="{% url 'home' %}">django_structlog_demo_project</a>
            </li>
            <li class="nav-item active">
              <a class="nav-link" href="{% url 'home' %}">Home</a>
            </li>
            <li class="nav-item">
              <a class="nav-link" href="{% url 'about' %}">About</a>
            </li>
            {% if request.user.is_authenticated %}
              <li class="nav-item">
                {# URL provided by django-allauth/account/urls.py #}
                <a class="nav-link" href="{% url 'users:detail' request.user.username  %}">{% trans "My Profile" %}</a>
              </li>
              <li class="nav-item">
                {# URL provided by django-allauth/account/urls.py #}
                <a class="nav-link" href="{% url 'account_logout' %}">{% trans "Sign Out" %}</a>
              </li>
            {% else %}
              <li class="nav-item">
                {# URL provided by django-allauth/account/urls.py #}
                <a id="sign-up-link" class="nav-link" href="{% url 'account_signup' %}">{% trans "Sign Up" %}</a>
              </li>
              <li class="nav-item">
                {# URL provided by django-allauth/account/urls.py #}
                <a id="log-in-link" class="nav-link" href="{% url 'account_login' %}">{% trans "Sign In" %}</a>
              </li>
            {% endif %}
          </ul>
        </div>
      </nav>

    </div>

    <div class="container">

      {% if messages %}
          {% for message in messages %}
              <div class="alert {% if message.tags %}alert-{{ message.tags }}{% endif %}">{{ message }}</div>
          {% endfor %}
      {% endif %}

      {% block content %}
        <p>Use this document as a way to quick start any new project.</p>
      {% endblock content %}

    </div> <!-- /container -->

    {% block modal %}{% endblock modal %}

    <!-- Le javascript
    ================================================== -->
    <!-- Placed at the end of the document so the pages load faster -->
    {% block javascript %}

    <!-- Bootstrap JS and its dependencies-->
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/js/bootstrap.bundle.min.js" integrity="sha384-ka7Sk0Gln4gmtz2MlQnikT1wXgYsOg+OMhuP+IlRH9sENBO0LRn5q+8nbTov4+1p" crossorigin="anonymous"></script>

    <!-- Your stuff: Third-party javascript libraries go here -->

    <!-- place project specific Javascript in this file -->

    <script src="{% static 'js/project.js' %}"></script>


    {% endblock javascript %}
  </body>
</html>

